@@ -31,6 +31,9 @@
 #include <sys/socket.h>
 #include <sys/types.h>
 
+#include <string.h>		/* For strstr */
+#include <stdio.h>		/* For printf */
+
 #include <irda.h>
 
 #include "irdadump.h"
@@ -83,13 +86,16 @@ inline void parse_iriap_command(GNetBuf *buf, GString *str, guint8 slsap_sel)
 			last_ias.obex = 1;
 		else
 			last_ias.obex = 0;
-#if 0
 		/* Check if this is a IrCOMM lookup */
-		if (strstr(name, "IrCOMM"))
+		if (strstr(name, "IrCOMM") || strstr(name, "IrLPT"))
 			last_ias.ircomm = 1;
 		else
 			last_ias.ircomm = 0;
-#endif	       
+		/* Check if this is a IrNET lookup */
+		if (strstr(name, "IrNet"))
+			last_ias.irnet = 1;
+		else
+			last_ias.irnet = 0;
 		break;
 	default:
 		break;
@@ -166,11 +172,10 @@ void parse_irias_value(GNetBuf *buf, GString *str)
 		value_len = tmp_cpu16;
 		
 		g_netbuf_pull(buf, n);
-#if 0
-		if (last_ias.ircomm)
-			parse_ircomm_connect(buf, str);
-		else
-#endif
+		if (last_ias.ircomm) {
+			g_string_append(str, "\n\tIrCOMM Parameters ");
+			parse_ircomm_params(value_len, buf, str);
+		} else
 			g_string_append(str, "N/A ");
 		break;
 	default:
@@ -187,7 +192,6 @@ void parse_irias_value(GNetBuf *buf, GString *str)
  */
 inline void parse_iriap_response(GNetBuf *buf, GString *str, guint8 dlsap_sel)
 {
-	int n=0;
 	guint8 opcode, rsp;
 
 	opcode = buf->data[0] & ~IAP_LST; /* Mask away LST bit */
@@ -223,16 +227,79 @@ inline void parse_iriap_response(GNetBuf *buf, GString *str, guint8 dlsap_sel)
 	}
 }
 
+/*
+ * Function parse_irttp (buf, str)
+ *
+ *    Parse IrTTP data frame
+ *
+ */
+inline void parse_irnet(GNetBuf *buf, GString *str) 
+{
+	g_string_sprintfa(str, "IrNET ");
+	/* If you want to hook PPP frame decoding, you can do it here.
+	 * I don't think it's worth it, PPP has debug capabilities.
+	 * Jean II */
+}
+
+/*
+ * Function parse_irttp (buf, str)
+ *
+ *    Parse IrTTP data frame
+ *
+ */
+inline void parse_irttp(GNetBuf *buf, GString *str) 
+{
+	g_string_sprintfa(str, "TTP credits=%d ", buf->data[0] & 0x7f);
+
+	if (buf->data[0] & 0x80)
+		g_string_append(str, "More ");
+
+	/* Remove TTP header */
+	g_netbuf_pull(buf, 1);
+}
+
+/*
+ * Function parse_irttp_connect (buf, str)
+ *
+ *    Parse IrTTP connect frame
+ *
+ */
+inline void parse_irttp_connect(GNetBuf *buf, GString *str)
+{
+	guint8 plen, pi, pl;
+	guint16 tmp_cpu;
+
+	g_string_sprintfa(str, "TTP credits=%d ", buf->data[0] & 0x7f);
+
+	if (buf->data[0] & 0x80) {
+		plen = buf->data[1];
+		pi   = buf->data[2];
+		pl   = buf->data[3];
+
+		memcpy(&tmp_cpu, buf->data+4, 2); /* Align value */
+		tmp_cpu = GUINT16_FROM_BE(tmp_cpu); /* Convert to host order */
+
+		/* Remove TTP parameters */
+		g_netbuf_pull(buf, plen);
+
+		g_string_sprintfa(str, "MaxSduSize=%d ", tmp_cpu);
+	}
+	/* Remove TTP header */
+	g_netbuf_pull(buf, 1);
+}
+
 /*
  * Function parse_irlmp (buf)
  *
  *    Parse IrLMP frame
  *
  */
-inline void parse_irlmp(GNetBuf *buf, GString *str, int type, int cmd)
+inline void parse_irlmp(GNetBuf *buf, GString *str,
+			guint8 caddr, int type, int cmd)
 {
 	guint8 slsap_sel, dlsap_sel;
 	int ctrl;
+	int rsvd;
 	int i;
 	
 	ctrl = buf->data[0] & CONTROL_BIT;
@@ -248,9 +315,10 @@ inline void parse_irlmp(GNetBuf *buf, GString *str, int type, int cmd)
 	/* Control or data? */
 	if (ctrl) {
 		ctrl = buf->data[0];
+		rsvd = buf->data[1];	/* reason/status/rsvd */
 
-		/* Remove IrLMP control header */
-		g_netbuf_pull(buf, 1);
+		/* Remove IrLMP control header (2 bytes) */
+		g_netbuf_pull(buf, 2);
 
 		switch (ctrl) {
 		case CONNECT_CMD:
@@ -262,6 +330,7 @@ inline void parse_irlmp(GNetBuf *buf, GString *str, int type, int cmd)
 					return;
 				}
 				conn[i].valid = 1;
+				conn[i].caddr = caddr;
 				if (type) {
 					conn[i].slsap_sel = slsap_sel;
 					conn[i].dlsap_sel = dlsap_sel;
@@ -271,13 +340,18 @@ inline void parse_irlmp(GNetBuf *buf, GString *str, int type, int cmd)
 				}
 				conn[i].ttp = last_ias.ttp;
 				conn[i].obex = last_ias.obex;
-			
+				conn[i].ircomm = last_ias.ircomm;
+				conn[i].irnet = last_ias.irnet;
+
 				if (conn[i].ttp)
 					parse_irttp_connect(buf, str);
-#if 0
 				if (conn[i].ircomm)
 					parse_ircomm_connect(buf, str);
-#endif
+				if (conn[i].irnet)
+					parse_irnet(buf, str);
+			} else {
+				if((config_force_ttp) && (dlsap_sel != 0x00))
+					parse_irttp_connect(buf, str);
 			}
 			break;
 		case CONNECT_RSP:
@@ -289,12 +363,14 @@ inline void parse_irlmp(GNetBuf *buf, GString *str, int type, int cmd)
 			if (i != -1) {
 				if (conn[i].ttp)
 					parse_irttp_connect(buf, str);
-#if 0
 				if (conn[i].ircomm)
 					parse_ircomm_connect(buf, str);
-#endif
+				if (conn[i].irnet)
+					parse_irnet(buf, str);
+			} else {
+				if((config_force_ttp) && (slsap_sel != 0x00))
+					parse_irttp_connect(buf, str);
 			}
-			
 			break;
 		case DISCONNECT:
 			g_string_append(str, "DISC ");
@@ -327,11 +403,20 @@ inline void parse_irlmp(GNetBuf *buf, GString *str, int type, int cmd)
 			if (conn[i].valid && conn[i].ttp)
 				parse_irttp(buf, str);
 			if (conn[i].valid && conn[i].obex)
-				parse_obex(&conn[i], buf, str, cmd);
-#if 0
-			if (conn[i].valid && conn[i].ircomm)
-				parse_ircomm(&conn[i], buf, str);
-#endif
+				parse_obex(buf, str, cmd);
+			if (conn[i].valid && conn[i].ircomm) {
+				if (conn[i].ttp)
+					parse_ircomm_ttp(buf, str);
+				else
+					parse_ircomm_lmp(buf, str);
+			}
+			if (conn[i].valid && conn[i].irnet)
+				parse_irnet(buf, str);
+		}
+		/* Not IAS and unknow connection */
+		if((dlsap_sel != 0x00) && (slsap_sel != 0x00) && (i == -1)) {
+			if(config_force_ttp)
+				parse_irttp(buf, str);
 		}
 	}
 }
@@ -347,7 +432,6 @@ inline void parse_ui_irlmp(GNetBuf *buf, GString *str, int type)
 {
 	guint8 slsap_sel, dlsap_sel;
 	int ctrl;
-	int i;
 	
 	ctrl = buf->data[0] & CONTROL_BIT;
 	dlsap_sel = buf->data[0] & LSAP_MASK;
@@ -374,7 +458,7 @@ inline void parse_ui_irlmp(GNetBuf *buf, GString *str, int type)
 		g_netbuf_pull(buf, 1);
 
 		/* Decode Obex stuff - no connection, always a command */
-		parse_obex(NULL, buf, str, 1);
+		parse_obex(buf, str, 1);
 	      }
 	  }
 }
